(*
 * ocamlsdl - Objective Caml interface for the SDL library
 * Copyright (C) 1999 Jean-Christophe FILLIATRE
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 *)

(* $Id: mixer.ml,v 1.4 2000/04/09 13:29:49 marche Exp $ *)

exception Mix_failure of string

let _ = Callback.register_exception "Mix_failure" (Mix_failure "")

type format = int
let format_AUDIO_U8	= 0x0008  (* Unsigned 8-bit samples *)
let format_AUDIO_S8	= 0x8008  (* Signed 8-bit samples *)
let format_AUDIO_U16LSB	= 0x0010  (* Unsigned 16-bit samples *)
let format_AUDIO_S16LSB	= 0x8010  (* Signed 16-bit samples *)
let format_AUDIO_U16MSB	= 0x1010  (* As above, but big-endian byte order *)
let format_AUDIO_S16MSB	= 0x9010  (* As above, but big-endian byte order *)
let format_AUDIO_U16	= format_AUDIO_U16LSB
let format_AUDIO_S16	= format_AUDIO_S16LSB

external open_audio : int -> format -> int -> int -> unit
  = "mixerstub_open_audio"
    (* [open_audio frequency format channels chunksize] *)

external close_audio : unit -> unit
  = "mixerstub_close_audio"


type music

external load_music : string -> music
  = "mixerstub_load_music"

external play_music : music -> int -> int
  = "mixerstub_play_music"
    (* returns the channel on which it is played *)

external playing_music : unit -> bool
  = "mixerstub_playing_music"

external fade_out_music : int -> unit
  = "mixerstub_fade_out_music"

external halt_music : unit -> unit
  = "mixerstub_halt_music"

external free_music : music -> unit
  = "mixerstub_free_music"


type chunk

external load_wav : string -> chunk
  = "mixerstub_load_wav"

external play_channel : int -> chunk -> int -> int
  = "mixerstub_play_channel"
    (* [play_channel channel chunk loops]. 
       returns the channel on which it is played *)

external playing : int -> bool
  = "mixerstub_playing"
    (* check all if -1 *)

external halt_channel : int -> unit
  = "mixerstub_halt_channel"
    (* all if -1 *)

external free_chunk : chunk -> unit
  = "mixerstub_free_chunk"


