/*
 * ocamlsdl - Objective Caml interface for the SDL library
 * Copyright (C) 1999 Jean-Christophe FILLIATRE
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 */

/* $Id: mixer_stub.c,v 1.5 2000/04/09 13:29:50 marche Exp $ */

#include <stdio.h>
#include "SDL_mixer.h" 
#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/fail.h>
#include <caml/callback.h>

void mix_raise_failure() {
  raise_with_string(*caml_named_value("Mix_failure"), Mix_GetError());
}

value mixerstub_open_audio(value vfreq, value vformat, value vch, value vs) {
  int frequency = Int_val(vfreq);
  int format = Int_val(vformat);
  int channels = Int_val(vch);
  int chunksize = Int_val(vs);
  int result;

  if (Mix_OpenAudio(frequency, format, channels, chunksize) < 0)
    mix_raise_failure();
  return Val_unit;
}

value mixerstub_close_audio(value u) {
  Mix_CloseAudio();
  return Val_unit;
}


value mixerstub_load_music(value vfile) {
  char * file = String_val(vfile);
  Mix_Music * m;

  if ((m = Mix_LoadMUS(file)) == NULL) mix_raise_failure();
  return (value) m;
}

value mixerstub_play_music(value m,value loops) {
  int ch;

  ch = Mix_PlayMusic((Mix_Music *) m, Int_val(loops));
  return Val_int(ch);
}

value mixerstub_playing_music(value u) {
  return Val_bool(Mix_PlayingMusic());
}

value mixerstub_fade_out_music(value ms) {
  Mix_FadeOutMusic(Int_val(ms));
  return Val_unit;
}

value mixerstub_halt_music(value u) {
  Mix_HaltMusic();
  return Val_unit;
}

value mixerstub_free_music(value m) {
  Mix_FreeMusic((Mix_Music *) m);
  return Val_unit;
}

value mixerstub_load_wav(value vfile) {
  char * file = String_val(vfile);
  Mix_Chunk * c;

  if ((c = Mix_LoadWAV(file)) == NULL) mix_raise_failure();
  return (value) c;
}

value mixerstub_play_channel(value vch, value c, value vl) {
  int channel = Int_val(vch);
  int loops = Int_val(vl);
  int result;
  
  result = Mix_PlayChannel(channel, (Mix_Chunk *) c, loops);
  return Val_int(result);
}

value mixerstub_playing(value vch) {
  int channel = Int_val(vch);

  return Val_bool(Mix_Playing(channel));
}

value mixerstub_halt_channel(value vch) {
  int channel = Int_val(vch);

  Mix_HaltChannel(channel);
  return Val_unit;
}

value mixerstub_free_chunk(value c) {
  Mix_FreeChunk((Mix_Chunk *) c);
  return Val_unit;
}
