
{
  open Lexing

  let outs = print_string
  let outc = print_char

  let table =
    let ht = Hashtbl.create 97 in
    List.iter (fun (a,b) -> Hashtbl.add ht a b)
      [
	"glViewport", "GlDraw.viewport "; 
	"glClearColor", "GlClear.color ";
	"glClearDepth", "GlClear.depth ";
	"glDepthFunc", "GlFunc.depth_func ";
	"glEnable", "Gl.enable ";
	"GL_LESS", "`less";
	"GL_DEPTH_TEST", "`depth_test";
	"glShadeModel", " GlDraw.shade_model ";
	"GL_SMOOTH", "`smooth";
	"glMatrixMode", "GlMat.mode ";
	"GL_PROJECTION", "`projection";
	"glLoadIdentity", "GlMat.load_identity ";
	"gluPerspective", "GluMat.perspective ";
	"GL_MODELVIEW", "`modelview";
	"glClear", "GlClear.clear ";
	"GL_COLOR_BUFFER_BIT", "`color";
	"GL_DEPTH_BUFFER_BIT", "`depth";
	"glTranslatef", "GlMat.translate3 ";
	"glBegin", "GlDraw.begins ";
	"GL_POLYGON", "`polygon";
	"glVertex3f", "GlDraw.vertex3 ";
	"glEnd", "GlDraw.ends ";
	"GL_QUADS", "`quads";
	"SDL_GL_SwapBuffers", "Sdl.GL.swap_buffers ";
	"glColor3f", "GlDraw.color ";
	"glRotatef", "GlMat.rotate3 ";
	"glTexParameteri", "GlTex.parameter";
	"GL_TEXTURE_2D", "`texture_2d";
	"GL_TEXTURE_MAG_FILTER", "`mag_filter";
	"GL_TEXTURE_MIN_FILTER", "`linear";
	"glTexImage2D", "GlTex.image2d";
	"glTexCoord2f", "GlTex.coord2";
      ];
    fun s -> try Hashtbl.find ht s with Not_found -> s

}

let id = ['a'-'z' 'A'-'Z' '_'] ['a'-'z' 'A'-'Z' '_' '0'-'9']*

rule trans = parse
  | "/*" { outs "(*"; comment lexbuf; trans lexbuf }
  | "//" [^ '\n']* { outs "(*"; 
		     let s = lexeme lexbuf in
		     outs (String.sub s 2 (String.length s - 2));
		     outs "*)";
		     trans lexbuf }
  | ['0'-'9']+ '.' ['0'-'9']+ 'f' 
      { let s = lexeme lexbuf in
	outs (String.sub s 0 (String.length s - 1));
	trans lexbuf }
  | id { outs (table (lexeme lexbuf)); trans lexbuf }
  | eof { () }
  | _ { outs (lexeme lexbuf); trans lexbuf }
and comment = parse
  | "*/" { outs "*)" }
  | eof { () }
  | _ { outs (lexeme lexbuf); comment lexbuf }

{

  let f = Sys.argv.(1)
  let cin = open_in f
  let lb = from_channel cin
  let _ = trans lb
  let _ = close_in cin

}
