
open Sdl
open Video
open Event

open GlDraw

(* A general OpenGL initialization function.  Sets all of the initial parameters. *)
let init_gl width height =	        (* We call this right after our OpenGL window is created.*)
  GlDraw.viewport 0 0 width height;
  GlClear.color ~alpha:0.0 (0.0, 0.0, 0.0);		(* This Will Clear The Background Color To Black*)
  GlClear.depth (1.0);				(* Enables Clearing Of The Depth Buffer*)
  GlFunc.depth_func (`less);				(* The Type Of Depth Test To Do*)
  Gl.enable (`depth_test);			(* Enables Depth Testing*)
  GlDraw.shade_model (`smooth);			(* Enables Smooth Color Shading*)

  GlMat.mode (`projection);
  GlMat.load_identity ();				(* Reset The Projection Matrix*)

  GluMat.perspective ~fovy:45.0 ~aspect:(float width /. float height) 
    ~z:(0.1, 100.0);

  GlMat.mode (`modelview)


let rtri = ref 0.0
let rquad = ref 0.0

(* The main drawing function. *)
let draw_gl_scene () =

  GlClear.clear [`color; `depth];		(* Clear The Screen And The Depth Buffer*)
  GlMat.load_identity ();				(* Reset The View*)

  GlMat.translate3 (-1.5,0.0,-6.0);		(* Move Left 1.5 Units And Into The Screen 6.0*)
	
  GlMat.rotate3 !rtri (0.0,1.0,0.0);          (* Rotate The Triangle On The Y axis *)
  (* draw a triangle (in smooth coloring mode)*)
  GlDraw.begins (`polygon);                             (* start drawing a polyg
on*)

  (* front face of pyramid*)
  GlDraw.color (1.0,0.0,0.0);                   (* Set The Color To Red*)
  GlDraw.vertex3 (0.0, 1.0, 0.0);                       (* Top of triangle (front)*)
  GlDraw.color (0.0,1.0,0.0);                   (* Set The Color To Green*)
  GlDraw.vertex3 (-1.0,-1.0, 1.0);              (* left of triangle (front)*)
  GlDraw.color (0.0,0.0,1.0);                   (* Set The Color To Blue*)
  GlDraw.vertex3 (1.0,-1.0, 1.0);                       (* right of traingle (front)    *)

  (* right face of pyramid*)
  GlDraw.color (1.0,0.0,0.0);                   (* Red*)
  GlDraw.vertex3 ( 0.0, 1.0, 0.0);              (* Top Of Triangle (Right)*)
  GlDraw.color (0.0,0.0,1.0);                   (* Blue*)
  GlDraw.vertex3 ( 1.0,-1.0, 1.0);              (* Left Of Triangle (Right)*)
  GlDraw.color (0.0,1.0,0.0);                   (* Green*)
  GlDraw.vertex3 ( 1.0,-1.0, -1.0);             (* Right Of Triangle (Right)*)

  (* back face of pyramid*)
  GlDraw.color (1.0,0.0,0.0);                   (* Red*)
  GlDraw.vertex3 ( 0.0, 1.0, 0.0);              (* Top Of Triangle (Back)*)
  GlDraw.color (0.0,1.0,0.0);                   (* Green*)
  GlDraw.vertex3 ( 1.0,-1.0, -1.0);             (* Left Of Triangle (Back)*)
  GlDraw.color (0.0,0.0,1.0);                   (* Blue*)
  GlDraw.vertex3 (-1.0,-1.0, -1.0);             (* Right Of Triangle (Back)*)

  (* left face of pyramid.*)
  GlDraw.color (1.0,0.0,0.0);                   (* Red*)
  GlDraw.vertex3 ( 0.0, 1.0, 0.0);              (* Top Of Triangle (Left)*)
  GlDraw.color (0.0,0.0,1.0);                   (* Blue*)
  GlDraw.vertex3 (-1.0,-1.0,-1.0);              (* Left Of Triangle (Left)*)
  GlDraw.color (0.0,1.0,0.0);                   (* Green*)
  GlDraw.vertex3 (-1.0,-1.0, 1.0);              (* Right Of Triangle (Left)*)

  GlDraw.ends ();                                       (* Done Drawing The Pyramid*)

  GlMat.load_identity ();                               (* make sure we're no longer rotated.*)
  GlMat.translate3 (1.5,0.0,-7.0);              (* Move Right 3 Units, and back into the screen 7*)
        
  GlMat.rotate3 !rquad (1.0,1.0,1.0);            (* Rotate The Cube On X, Y, and Z*)

  (* draw a cube (6 quadrilaterals)*)
  GlDraw.begins (`quads);                               (* start drawing the cube.*)

    (* top of cube*)
  GlDraw.color (0.0,1.0,0.0);                   (* Set The Color To Blue*)
  GlDraw.vertex3 ( 1.0, 1.0,-1.0);              (* Top Right Of The Quad (Top)*)
  GlDraw.vertex3 (-1.0, 1.0,-1.0);              (* Top Left Of The Quad (Top)*)
  GlDraw.vertex3 (-1.0, 1.0, 1.0);              (* Bottom Left Of The Quad (Top)*)
  GlDraw.vertex3 ( 1.0, 1.0, 1.0);              (* Bottom Right Of The Quad (Top)*)

  (* bottom of cube*)
  GlDraw.color (1.0,0.5,0.0);                   (* Set The Color To Orange*)
  GlDraw.vertex3 ( 1.0,-1.0, 1.0);              (* Top Right Of The Quad (Bottom)*)
  GlDraw.vertex3 (-1.0,-1.0, 1.0);              (* Top Left Of The Quad (Bottom)*)
  GlDraw.vertex3 (-1.0,-1.0,-1.0);              (* Bottom Left Of The Quad (Bottom)*)
  GlDraw.vertex3 ( 1.0,-1.0,-1.0);              (* Bottom Right Of The Quad (Bottom)*)

  (* front of cube*)
  GlDraw.color (1.0,0.0,0.0);                   (* Set The Color To Red*)
  GlDraw.vertex3 ( 1.0, 1.0, 1.0);              (* Top Right Of The Quad (Front)*)
  GlDraw.vertex3 (-1.0, 1.0, 1.0);              (* Top Left Of The Quad (Front)*)
  GlDraw.vertex3 (-1.0,-1.0, 1.0);              (* Bottom Left Of The Quad (Front)*)
  GlDraw.vertex3 ( 1.0,-1.0, 1.0);              (* Bottom Right Of The Quad (Front)*)

  (* back of cube.*)
  GlDraw.color (1.0,1.0,0.0);                   (* Set The Color To Yellow*)
  GlDraw.vertex3 ( 1.0,-1.0,-1.0);              (* Top Right Of The Quad (Back)*)
  GlDraw.vertex3 (-1.0,-1.0,-1.0);              (* Top Left Of The Quad (Back)*)
  GlDraw.vertex3 (-1.0, 1.0,-1.0);              (* Bottom Left Of The Quad (Back)*)
  GlDraw.vertex3 ( 1.0, 1.0,-1.0);              (* Bottom Right Of The Quad (Back)*)

  (* left of cube*)
  GlDraw.color (0.0,0.0,1.0);                   (* Blue*)
  GlDraw.vertex3 (-1.0, 1.0, 1.0);              (* Top Right Of The Quad (Left)*)
  GlDraw.vertex3 (-1.0, 1.0,-1.0);              (* Top Left Of The Quad (Left)*)
  GlDraw.vertex3 (-1.0,-1.0,-1.0);              (* Bottom Left Of The Quad (Left)*)
  GlDraw.vertex3 (-1.0,-1.0, 1.0);              (* Bottom Right Of The Quad (Left)*)

  (* Right of cube*)
  GlDraw.color (1.0,0.0,1.0);                   (* Set The Color To Violet*)
  GlDraw.vertex3 ( 1.0, 1.0,-1.0);              (* Top Right Of The Quad (Right)*)
  GlDraw.vertex3 ( 1.0, 1.0, 1.0);              (* Top Left Of The Quad (Right)*)
  GlDraw.vertex3 ( 1.0,-1.0, 1.0);              (* Bottom Left Of The Quad (Right)*)
  GlDraw.vertex3 ( 1.0,-1.0,-1.0);              (* Bottom Right Of The Quad (Right)*)
  GlDraw.ends ();                                       (* Done Drawing The Cube*)

  rtri := !rtri +. 15.0;                                   (* Increase The Rotation Variable For The Triangle*)
  rquad := !rquad -. 15.0;                                  (* Decrease The Rotation Variable For The Quad *)

  (* swap buffers to display, since we're double buffered.*)
  Sdl.GL.swap_buffers ()

let main () =
  let count = ref 0 in
  let done_ = ref false in
  (* Initialize SDL for video output *)
  Sdl.init [VIDEO];
  let ticks0 = Timer.get_ticks () in
  (* Create a 640x480 OpenGL screen *)
  let screen = set_video_mode 640 480 0 [OPENGL] in
  (* Loop, drawing and checking events *)
  init_gl 640 480;
  while not !done_ do
    draw_gl_scene ();
    incr count;
    (* This could go in a separate function *)
    match poll_event () with
      | Some Quit -> done_ := true
      | Some (KeyDown k) ->
	  (match k.key_sym.key with
	     | 27 -> done_ := true
	     | _ -> ())
      | _ -> ()
  done;
  let ticks1 = Timer.get_ticks () in
  Printf.printf "%d %d %d\n" !count ticks0 ticks1; flush stdout;
  let fps = float !count *. 1000. /. float (ticks1 - ticks0) in
  Printf.printf "%f frames per second\n" fps; flush stdout

let _ = try main (); Sdl.quit () with e -> Sdl.quit (); raise e
